@@ -7,7 +7,6 @@ from requests.exceptions import ConnectionError
 import urllib.parse
 import requests
 
-
 from typing import Optional, List
 
 from pydantic import BaseModel, ValidationError, validator,   Field
@@ -158,11 +157,31 @@ class Collection(BaseModel):
     created_at: float = Field(description='The epoch timestamp when the collection was created.')
     updated_at: float = Field(description='The epoch timestamp when the collection was updated.')
        
-    def add(self, vector_id, vector_data):
-        session.post("/collections/%d/vectors/%d" % (self.id, vector_id), json={'vector': list(vector_data)})
+    def add(self, vector_data:[float], vector_id:int):
+        """
+        Add the specified vector_data to the collection and associate it with the specified vector_id.
+        A collection may not have more than one vector with a given vector_id.
+        Re-using a vector_id will over-write any existing  vector_id +  vector pair.
+        All vectors in a collection must share the same dimension.
+        """
+        vector_data = [float(x) for x in vector_data]
+        session.post(f"/collections/{self.id}/vectors/{vector_id}", json={'vector': vector_data})
         if not self.dimension:
             self.dimension = len(vector_data)  # record dimension
-            
+
+    def get(self, vector_id : int):
+        """
+        Return the vector with the specified vector_id from the collection
+        """
+        resp = session.get(f"/collections/{self.id}/vectors/{vector_id}")
+        return resp.json()['vector']
+        
+    def delete(self, vector_id : int):
+        """
+        Delete the vector with the specified vector_id from the collection
+        """        
+        session.delete(f"/collections/{self.id}/vectors/{vector_id}")
+        
     def create_index(self,
                      tag='latest',
                      target_library=IndexLibraries.hnswlib,
@@ -178,7 +197,7 @@ class Collection(BaseModel):
         resp = session.post("/collections/%d/index" % self.id, json=body)
         return Index(**resp.json(), collection=self)
                                                             
-    def delete(self):
+    def delete_collection(self):
         """
         Delete the collection.
         """
@@ -191,7 +210,7 @@ class Collection(BaseModel):
         self.dimension=None
 
 
-    def get_index(self, tag="latest"):
+    def get_index(self, tag:str="latest"):
         data = {'tag': tag}
         resp = session.get(f"/collections/{self.id}/index?{urllib.parse.urlencode(data)}")
         items = resp.json()['items']
@@ -212,7 +231,7 @@ class Collection(BaseModel):
         return object.__getattribute__(self, attr)        
 
 
-def create_collection(name):
+def create_collection(name:str):
     """
     Create a collection of the specified name
     Returns a Collection object.
@@ -223,10 +242,10 @@ def create_collection(name):
     return Collection(**resp.json())
 
 
-def collection(name):
+def collection(name:str):
     """
-    Get a collection by name, creating it if it doesn't already exist.
-    Returns a Collection Object
+    Get a collection by name.
+    Returns a Collection Object, or None if collection name is not found
     """
     data = {'name': name}
     resp = session.get(f"/collections?{urllib.parse.urlencode(data)}")
@@ -234,8 +253,10 @@ def collection(name):
     assert(len(items) <= 1)
     if len(items):
         return Collection(**items[0])
-    else:
-        return create_collection(name)
+
+def all_collections():
+    resp = session.get("/collections")
+    return [Collection(**i) for i in resp.json()['items']]
 
 
 class Index(BaseModel):
@@ -267,7 +288,7 @@ class Index(BaseModel):
             raise IndexError(self.build_status)
         
             
-    def hnswlib_index(self, max_elements=None):
+    def hnswlib_index(self, max_elements:int = None):
         self._wait_complete()
         if not max_elements:
             max_elements = self.count
